# pygpgme - a Python wrapper for the gpgme library
# Copyright (C) 2006  James Henstridge
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
PyGPGME, a Python wrapper for the GPGME library.

PyGPGME is a Python module that lets you sign, verify, encrypt and decrypt
messages using the OpenPGP format.

It is built on top of the GNU Privacy Guard and the GPGME library.

All interaction begins by creating a Context.
"""

from gpgme._gpgme import *

__version__ = '0.6'

DATA_ENCODING_NONE = DataEncoding.NONE
DATA_ENCODING_BINARY = DataEncoding.BINARY
DATA_ENCODING_BASE64 = DataEncoding.BASE64
DATA_ENCODING_ARMOR = DataEncoding.ARMOR

PK_RSA = PubkeyAlgo.RSA
PK_RSA_E = PubkeyAlgo.RSA_E
PK_RSA_S = PubkeyAlgo.RSA_S
PK_ELG_E = PubkeyAlgo.ELG_E
PK_DSA = PubkeyAlgo.DSA
PK_ELG = PubkeyAlgo.ELG

MD_NONE = HashAlgo.NONE
MD_MD5 = HashAlgo.MD5
MD_SHA1 = HashAlgo.SHA1
MD_RMD160 = HashAlgo.RMD160
MD_MD2 = HashAlgo.MD2
MD_TIGER = HashAlgo.TIGER
MD_HAVAL = HashAlgo.HAVAL
MD_SHA256 = HashAlgo.SHA256
MD_SHA384 = HashAlgo.SHA384
MD_SHA512 = HashAlgo.SHA512
MD_MD4 = HashAlgo.MD4
MD_CRC32 = HashAlgo.CRC32
MD_CRC32_RFC1510 = HashAlgo.CRC32_RFC1510
MD_CRC24_RFC2440 = HashAlgo.CRC24_RFC2440

SIG_MODE_NORMAL = SigMode.NORMAL
SIG_MODE_DETACH = SigMode.DETACH
SIG_MODE_CLEAR = SigMode.CLEAR

VALIDITY_UNKNOWN = Validity.UNKNOWN
VALIDITY_UNDEFINED = Validity.UNDEFINED
VALIDITY_NEVER = Validity.NEVER
VALIDITY_MARGINAL = Validity.MARGINAL
VALIDITY_FULL = Validity.FULL
VALIDITY_ULTIMATE = Validity.ULTIMATE

PROTOCOL_OpenPGP = Protocol.OpenPGP
PROTOCOL_CMS = Protocol.CMS

KEYLIST_MODE_LOCAL = KeylistMode.LOCAL
KEYLIST_MODE_EXTERN = KeylistMode.EXTERN
KEYLIST_MODE_SIGS = KeylistMode.SIGS
KEYLIST_MODE_VALIDATE = KeylistMode.VALIDATE

STATUS_EOF = Status.EOF
STATUS_ENTER = Status.ENTER
STATUS_LEAVE = Status.LEAVE
STATUS_ABORT = Status.ABORT
STATUS_GOODSIG = Status.GOODSIG
STATUS_BADSIG = Status.BADSIG
STATUS_ERRSIG = Status.ERRSIG
STATUS_BADARMOR = Status.BADARMOR
STATUS_RSA_OR_IDEA = Status.RSA_OR_IDEA
STATUS_KEYEXPIRED = Status.KEYEXPIRED
STATUS_KEYREVOKED = Status.KEYREVOKED
STATUS_TRUST_UNDEFINED = Status.TRUST_UNDEFINED
STATUS_TRUST_NEVER = Status.TRUST_NEVER
STATUS_TRUST_MARGINAL = Status.TRUST_MARGINAL
STATUS_TRUST_FULLY = Status.TRUST_FULLY
STATUS_TRUST_ULTIMATE = Status.TRUST_ULTIMATE
STATUS_SHM_INFO = Status.SHM_INFO
STATUS_SHM_GET = Status.SHM_GET
STATUS_SHM_GET_BOOL = Status.SHM_GET_BOOL
STATUS_SHM_GET_HIDDEN = Status.SHM_GET_HIDDEN
STATUS_NEED_PASSPHRASE = Status.NEED_PASSPHRASE
STATUS_VALIDSIG = Status.VALIDSIG
STATUS_SIG_ID = Status.SIG_ID
STATUS_ENC_TO = Status.ENC_TO
STATUS_NODATA = Status.NODATA
STATUS_BAD_PASSPHRASE = Status.BAD_PASSPHRASE
STATUS_NO_PUBKEY = Status.NO_PUBKEY
STATUS_NO_SECKEY = Status.NO_SECKEY
STATUS_NEED_PASSPHRASE_SYM = Status.NEED_PASSPHRASE_SYM
STATUS_DECRYPTION_FAILED = Status.DECRYPTION_FAILED
STATUS_DECRYPTION_OKAY = Status.DECRYPTION_OKAY
STATUS_MISSING_PASSPHRASE = Status.MISSING_PASSPHRASE
STATUS_GOOD_PASSPHRASE = Status.GOOD_PASSPHRASE
STATUS_GOODMDC = Status.GOODMDC
STATUS_BADMDC = Status.BADMDC
STATUS_ERRMDC = Status.ERRMDC
STATUS_IMPORTED = Status.IMPORTED
STATUS_IMPORT_OK = Status.IMPORT_OK
STATUS_IMPORT_PROBLEM = Status.IMPORT_PROBLEM
STATUS_IMPORT_RES = Status.IMPORT_RES
STATUS_FILE_START = Status.FILE_START
STATUS_FILE_DONE = Status.FILE_DONE
STATUS_FILE_ERROR = Status.FILE_ERROR
STATUS_BEGIN_DECRYPTION = Status.BEGIN_DECRYPTION
STATUS_END_DECRYPTION = Status.END_DECRYPTION
STATUS_BEGIN_ENCRYPTION = Status.BEGIN_ENCRYPTION
STATUS_END_ENCRYPTION = Status.END_ENCRYPTION
STATUS_DELETE_PROBLEM = Status.DELETE_PROBLEM
STATUS_GET_BOOL = Status.GET_BOOL
STATUS_GET_LINE = Status.GET_LINE
STATUS_GET_HIDDEN = Status.GET_HIDDEN
STATUS_GOT_IT = Status.GOT_IT
STATUS_PROGRESS = Status.PROGRESS
STATUS_SIG_CREATED = Status.SIG_CREATED
STATUS_SESSION_KEY = Status.SESSION_KEY
STATUS_NOTATION_NAME = Status.NOTATION_NAME
STATUS_NOTATION_DATA = Status.NOTATION_DATA
STATUS_POLICY_URL = Status.POLICY_URL
STATUS_BEGIN_STREAM = Status.BEGIN_STREAM
STATUS_END_STREAM = Status.END_STREAM
STATUS_KEY_CREATED = Status.KEY_CREATED
STATUS_USERID_HINT = Status.USERID_HINT
STATUS_UNEXPECTED = Status.UNEXPECTED
STATUS_INV_RECP = Status.INV_RECP
STATUS_NO_RECP = Status.NO_RECP
STATUS_ALREADY_SIGNED = Status.ALREADY_SIGNED
STATUS_SIGEXPIRED = Status.SIGEXPIRED
STATUS_EXPSIG = Status.EXPSIG
STATUS_EXPKEYSIG = Status.EXPKEYSIG
STATUS_TRUNCATED = Status.TRUNCATED
STATUS_ERROR = Status.ERROR
STATUS_NEWSIG = Status.NEWSIG
STATUS_REVKEYSIG = Status.REVKEYSIG

ENCRYPT_ALWAYS_TRUST = EncryptFlags.ALWAYS_TRUST

SIGSUM_VALID = Sigsum.VALID
SIGSUM_GREEN = Sigsum.GREEN
SIGSUM_RED = Sigsum.RED
SIGSUM_KEY_REVOKED = Sigsum.KEY_REVOKED
SIGSUM_KEY_EXPIRED = Sigsum.KEY_EXPIRED
SIGSUM_SIG_EXPIRED = Sigsum.SIG_EXPIRED
SIGSUM_KEY_MISSING = Sigsum.KEY_MISSING
SIGSUM_CRL_MISSING = Sigsum.CRL_MISSING
SIGSUM_CRL_TOO_OLD = Sigsum.CRL_TOO_OLD
SIGSUM_BAD_POLICY = Sigsum.BAD_POLICY
SIGSUM_SYS_ERROR = Sigsum.SYS_ERROR

IMPORT_NEW = Import.NEW
IMPORT_UID = Import.UID
IMPORT_SIG = Import.SIG
IMPORT_SUBKEY = Import.SUBKEY
IMPORT_SECRET = Import.SECRET

ERR_SOURCE_UNKNOWN = ErrSource.UNKNOWN
ERR_SOURCE_GCRYPT = ErrSource.GCRYPT
ERR_SOURCE_GPG = ErrSource.GPG
ERR_SOURCE_GPGSM = ErrSource.GPGSM
ERR_SOURCE_GPGAGENT = ErrSource.GPGAGENT
ERR_SOURCE_PINENTRY = ErrSource.PINENTRY
ERR_SOURCE_SCD = ErrSource.SCD
ERR_SOURCE_GPGME = ErrSource.GPGME
ERR_SOURCE_KEYBOX = ErrSource.KEYBOX
ERR_SOURCE_KSBA = ErrSource.KSBA
ERR_SOURCE_DIRMNGR = ErrSource.DIRMNGR
ERR_SOURCE_GSTI = ErrSource.GSTI
ERR_SOURCE_USER_1 = ErrSource.USER_1
ERR_SOURCE_USER_2 = ErrSource.USER_2
ERR_SOURCE_USER_3 = ErrSource.USER_3
ERR_SOURCE_USER_4 = ErrSource.USER_4

ERR_NO_ERROR = ErrCode.NO_ERROR
ERR_GENERAL = ErrCode.GENERAL
ERR_UNKNOWN_PACKET = ErrCode.UNKNOWN_PACKET
ERR_UNKNOWN_VERSION = ErrCode.UNKNOWN_VERSION
ERR_PUBKEY_ALGO = ErrCode.PUBKEY_ALGO
ERR_DIGEST_ALGO = ErrCode.DIGEST_ALGO
ERR_BAD_PUBKEY = ErrCode.BAD_PUBKEY
ERR_BAD_SECKEY = ErrCode.BAD_SECKEY
ERR_BAD_SIGNATURE = ErrCode.BAD_SIGNATURE
ERR_NO_PUBKEY = ErrCode.NO_PUBKEY
ERR_CHECKSUM = ErrCode.CHECKSUM
ERR_BAD_PASSPHRASE = ErrCode.BAD_PASSPHRASE
ERR_CIPHER_ALGO = ErrCode.CIPHER_ALGO
ERR_KEYRING_OPEN = ErrCode.KEYRING_OPEN
ERR_INV_PACKET = ErrCode.INV_PACKET
ERR_INV_ARMOR = ErrCode.INV_ARMOR
ERR_NO_USER_ID = ErrCode.NO_USER_ID
ERR_NO_SECKEY = ErrCode.NO_SECKEY
ERR_WRONG_SECKEY = ErrCode.WRONG_SECKEY
ERR_BAD_KEY = ErrCode.BAD_KEY
ERR_COMPR_ALGO = ErrCode.COMPR_ALGO
ERR_NO_PRIME = ErrCode.NO_PRIME
ERR_NO_ENCODING_METHOD = ErrCode.NO_ENCODING_METHOD
ERR_NO_ENCRYPTION_SCHEME = ErrCode.NO_ENCRYPTION_SCHEME
ERR_NO_SIGNATURE_SCHEME = ErrCode.NO_SIGNATURE_SCHEME
ERR_INV_ATTR = ErrCode.INV_ATTR
ERR_NO_VALUE = ErrCode.NO_VALUE
ERR_NOT_FOUND = ErrCode.NOT_FOUND
ERR_VALUE_NOT_FOUND = ErrCode.VALUE_NOT_FOUND
ERR_SYNTAX = ErrCode.SYNTAX
ERR_BAD_MPI = ErrCode.BAD_MPI
ERR_INV_PASSPHRASE = ErrCode.INV_PASSPHRASE
ERR_SIG_CLASS = ErrCode.SIG_CLASS
ERR_RESOURCE_LIMIT = ErrCode.RESOURCE_LIMIT
ERR_INV_KEYRING = ErrCode.INV_KEYRING
ERR_TRUSTDB = ErrCode.TRUSTDB
ERR_BAD_CERT = ErrCode.BAD_CERT
ERR_INV_USER_ID = ErrCode.INV_USER_ID
ERR_UNEXPECTED = ErrCode.UNEXPECTED
ERR_TIME_CONFLICT = ErrCode.TIME_CONFLICT
ERR_KEYSERVER = ErrCode.KEYSERVER
ERR_WRONG_PUBKEY_ALGO = ErrCode.WRONG_PUBKEY_ALGO
ERR_TRIBUTE_TO_D_A = ErrCode.TRIBUTE_TO_D_A
ERR_WEAK_KEY = ErrCode.WEAK_KEY
ERR_INV_KEYLEN = ErrCode.INV_KEYLEN
ERR_INV_ARG = ErrCode.INV_ARG
ERR_BAD_URI = ErrCode.BAD_URI
ERR_INV_URI = ErrCode.INV_URI
ERR_NETWORK = ErrCode.NETWORK
ERR_UNKNOWN_HOST = ErrCode.UNKNOWN_HOST
ERR_SELFTEST_FAILED = ErrCode.SELFTEST_FAILED
ERR_NOT_ENCRYPTED = ErrCode.NOT_ENCRYPTED
ERR_NOT_PROCESSED = ErrCode.NOT_PROCESSED
ERR_UNUSABLE_PUBKEY = ErrCode.UNUSABLE_PUBKEY
ERR_UNUSABLE_SECKEY = ErrCode.UNUSABLE_SECKEY
ERR_INV_VALUE = ErrCode.INV_VALUE
ERR_BAD_CERT_CHAIN = ErrCode.BAD_CERT_CHAIN
ERR_MISSING_CERT = ErrCode.MISSING_CERT
ERR_NO_DATA = ErrCode.NO_DATA
ERR_BUG = ErrCode.BUG
ERR_NOT_SUPPORTED = ErrCode.NOT_SUPPORTED
ERR_INV_OP = ErrCode.INV_OP
ERR_TIMEOUT = ErrCode.TIMEOUT
ERR_INTERNAL = ErrCode.INTERNAL
ERR_EOF_GCRYPT = ErrCode.EOF_GCRYPT
ERR_INV_OBJ = ErrCode.INV_OBJ
ERR_TOO_SHORT = ErrCode.TOO_SHORT
ERR_TOO_LARGE = ErrCode.TOO_LARGE
ERR_NO_OBJ = ErrCode.NO_OBJ
ERR_NOT_IMPLEMENTED = ErrCode.NOT_IMPLEMENTED
ERR_CONFLICT = ErrCode.CONFLICT
ERR_INV_CIPHER_MODE = ErrCode.INV_CIPHER_MODE
ERR_INV_FLAG = ErrCode.INV_FLAG
ERR_INV_HANDLE = ErrCode.INV_HANDLE
ERR_TRUNCATED = ErrCode.TRUNCATED
ERR_INCOMPLETE_LINE = ErrCode.INCOMPLETE_LINE
ERR_INV_RESPONSE = ErrCode.INV_RESPONSE
ERR_NO_AGENT = ErrCode.NO_AGENT
ERR_AGENT = ErrCode.AGENT
ERR_INV_DATA = ErrCode.INV_DATA
ERR_ASSUAN_SERVER_FAULT = ErrCode.ASSUAN_SERVER_FAULT
ERR_ASSUAN = ErrCode.ASSUAN
ERR_INV_SESSION_KEY = ErrCode.INV_SESSION_KEY
ERR_INV_SEXP = ErrCode.INV_SEXP
ERR_UNSUPPORTED_ALGORITHM = ErrCode.UNSUPPORTED_ALGORITHM
ERR_NO_PIN_ENTRY = ErrCode.NO_PIN_ENTRY
ERR_PIN_ENTRY = ErrCode.PIN_ENTRY
ERR_BAD_PIN = ErrCode.BAD_PIN
ERR_INV_NAME = ErrCode.INV_NAME
ERR_BAD_DATA = ErrCode.BAD_DATA
ERR_INV_PARAMETER = ErrCode.INV_PARAMETER
ERR_WRONG_CARD = ErrCode.WRONG_CARD
ERR_NO_DIRMNGR = ErrCode.NO_DIRMNGR
ERR_DIRMNGR = ErrCode.DIRMNGR
ERR_CERT_REVOKED = ErrCode.CERT_REVOKED
ERR_NO_CRL_KNOWN = ErrCode.NO_CRL_KNOWN
ERR_CRL_TOO_OLD = ErrCode.CRL_TOO_OLD
ERR_LINE_TOO_LONG = ErrCode.LINE_TOO_LONG
ERR_NOT_TRUSTED = ErrCode.NOT_TRUSTED
ERR_CANCELED = ErrCode.CANCELED
ERR_BAD_CA_CERT = ErrCode.BAD_CA_CERT
ERR_CERT_EXPIRED = ErrCode.CERT_EXPIRED
ERR_CERT_TOO_YOUNG = ErrCode.CERT_TOO_YOUNG
ERR_UNSUPPORTED_CERT = ErrCode.UNSUPPORTED_CERT
ERR_UNKNOWN_SEXP = ErrCode.UNKNOWN_SEXP
ERR_UNSUPPORTED_PROTECTION = ErrCode.UNSUPPORTED_PROTECTION
ERR_CORRUPTED_PROTECTION = ErrCode.CORRUPTED_PROTECTION
ERR_AMBIGUOUS_NAME = ErrCode.AMBIGUOUS_NAME
ERR_CARD = ErrCode.CARD
ERR_CARD_RESET = ErrCode.CARD_RESET
ERR_CARD_REMOVED = ErrCode.CARD_REMOVED
ERR_INV_CARD = ErrCode.INV_CARD
ERR_CARD_NOT_PRESENT = ErrCode.CARD_NOT_PRESENT
ERR_NO_PKCS15_APP = ErrCode.NO_PKCS15_APP
ERR_NOT_CONFIRMED = ErrCode.NOT_CONFIRMED
ERR_CONFIGURATION = ErrCode.CONFIGURATION
ERR_NO_POLICY_MATCH = ErrCode.NO_POLICY_MATCH
ERR_INV_INDEX = ErrCode.INV_INDEX
ERR_INV_ID = ErrCode.INV_ID
ERR_NO_SCDAEMON = ErrCode.NO_SCDAEMON
ERR_SCDAEMON = ErrCode.SCDAEMON
ERR_UNSUPPORTED_PROTOCOL = ErrCode.UNSUPPORTED_PROTOCOL
ERR_BAD_PIN_METHOD = ErrCode.BAD_PIN_METHOD
ERR_CARD_NOT_INITIALIZED = ErrCode.CARD_NOT_INITIALIZED
ERR_UNSUPPORTED_OPERATION = ErrCode.UNSUPPORTED_OPERATION
ERR_WRONG_KEY_USAGE = ErrCode.WRONG_KEY_USAGE
ERR_NOTHING_FOUND = ErrCode.NOTHING_FOUND
ERR_WRONG_BLOB_TYPE = ErrCode.WRONG_BLOB_TYPE
ERR_MISSING_VALUE = ErrCode.MISSING_VALUE
ERR_HARDWARE = ErrCode.HARDWARE
ERR_PIN_BLOCKED = ErrCode.PIN_BLOCKED
ERR_USE_CONDITIONS = ErrCode.USE_CONDITIONS
ERR_PIN_NOT_SYNCED = ErrCode.PIN_NOT_SYNCED
ERR_INV_CRL = ErrCode.INV_CRL
ERR_BAD_BER = ErrCode.BAD_BER
ERR_INV_BER = ErrCode.INV_BER
ERR_ELEMENT_NOT_FOUND = ErrCode.ELEMENT_NOT_FOUND
ERR_IDENTIFIER_NOT_FOUND = ErrCode.IDENTIFIER_NOT_FOUND
ERR_INV_TAG = ErrCode.INV_TAG
ERR_INV_LENGTH = ErrCode.INV_LENGTH
ERR_INV_KEYINFO = ErrCode.INV_KEYINFO
ERR_UNEXPECTED_TAG = ErrCode.UNEXPECTED_TAG
ERR_NOT_DER_ENCODED = ErrCode.NOT_DER_ENCODED
ERR_NO_CMS_OBJ = ErrCode.NO_CMS_OBJ
ERR_INV_CMS_OBJ = ErrCode.INV_CMS_OBJ
ERR_UNKNOWN_CMS_OBJ = ErrCode.UNKNOWN_CMS_OBJ
ERR_UNSUPPORTED_CMS_OBJ = ErrCode.UNSUPPORTED_CMS_OBJ
ERR_UNSUPPORTED_ENCODING = ErrCode.UNSUPPORTED_ENCODING
ERR_UNSUPPORTED_CMS_VERSION = ErrCode.UNSUPPORTED_CMS_VERSION
ERR_UNKNOWN_ALGORITHM = ErrCode.UNKNOWN_ALGORITHM
ERR_INV_ENGINE = ErrCode.INV_ENGINE
ERR_PUBKEY_NOT_TRUSTED = ErrCode.PUBKEY_NOT_TRUSTED
ERR_DECRYPT_FAILED = ErrCode.DECRYPT_FAILED
ERR_KEY_EXPIRED = ErrCode.KEY_EXPIRED
ERR_SIG_EXPIRED = ErrCode.SIG_EXPIRED
ERR_ENCODING_PROBLEM = ErrCode.ENCODING_PROBLEM
ERR_INV_STATE = ErrCode.INV_STATE
ERR_DUP_VALUE = ErrCode.DUP_VALUE
ERR_MISSING_ACTION = ErrCode.MISSING_ACTION
ERR_MODULE_NOT_FOUND = ErrCode.MODULE_NOT_FOUND
ERR_INV_OID_STRING = ErrCode.INV_OID_STRING
ERR_INV_TIME = ErrCode.INV_TIME
ERR_INV_CRL_OBJ = ErrCode.INV_CRL_OBJ
ERR_UNSUPPORTED_CRL_VERSION = ErrCode.UNSUPPORTED_CRL_VERSION
ERR_INV_CERT_OBJ = ErrCode.INV_CERT_OBJ
ERR_UNKNOWN_NAME = ErrCode.UNKNOWN_NAME
ERR_LOCALE_PROBLEM = ErrCode.LOCALE_PROBLEM
ERR_NOT_LOCKED = ErrCode.NOT_LOCKED
ERR_PROTOCOL_VIOLATION = ErrCode.PROTOCOL_VIOLATION
ERR_INV_MAC = ErrCode.INV_MAC
ERR_INV_REQUEST = ErrCode.INV_REQUEST
ERR_BUFFER_TOO_SHORT = ErrCode.BUFFER_TOO_SHORT
ERR_SEXP_INV_LEN_SPEC = ErrCode.SEXP_INV_LEN_SPEC
ERR_SEXP_STRING_TOO_LONG = ErrCode.SEXP_STRING_TOO_LONG
ERR_SEXP_UNMATCHED_PAREN = ErrCode.SEXP_UNMATCHED_PAREN
ERR_SEXP_NOT_CANONICAL = ErrCode.SEXP_NOT_CANONICAL
ERR_SEXP_BAD_CHARACTER = ErrCode.SEXP_BAD_CHARACTER
ERR_SEXP_BAD_QUOTATION = ErrCode.SEXP_BAD_QUOTATION
ERR_SEXP_ZERO_PREFIX = ErrCode.SEXP_ZERO_PREFIX
ERR_SEXP_NESTED_DH = ErrCode.SEXP_NESTED_DH
ERR_SEXP_UNMATCHED_DH = ErrCode.SEXP_UNMATCHED_DH
ERR_SEXP_UNEXPECTED_PUNC = ErrCode.SEXP_UNEXPECTED_PUNC
ERR_SEXP_BAD_HEX_CHAR = ErrCode.SEXP_BAD_HEX_CHAR
ERR_SEXP_ODD_HEX_NUMBERS = ErrCode.SEXP_ODD_HEX_NUMBERS
ERR_SEXP_BAD_OCT_CHAR = ErrCode.SEXP_BAD_OCT_CHAR
ERR_USER_1 = ErrCode.USER_1
ERR_USER_2 = ErrCode.USER_2
ERR_USER_3 = ErrCode.USER_3
ERR_USER_4 = ErrCode.USER_4
ERR_USER_5 = ErrCode.USER_5
ERR_USER_6 = ErrCode.USER_6
ERR_USER_7 = ErrCode.USER_7
ERR_USER_8 = ErrCode.USER_8
ERR_USER_9 = ErrCode.USER_9
ERR_USER_10 = ErrCode.USER_10
ERR_USER_11 = ErrCode.USER_11
ERR_USER_12 = ErrCode.USER_12
ERR_USER_13 = ErrCode.USER_13
ERR_USER_14 = ErrCode.USER_14
ERR_USER_15 = ErrCode.USER_15
ERR_USER_16 = ErrCode.USER_16
ERR_UNKNOWN_ERRNO = ErrCode.UNKNOWN_ERRNO
ERR_EOF = ErrCode.EOF
ERR_E2BIG = ErrCode.E2BIG
ERR_EACCES = ErrCode.EACCES
ERR_EADDRINUSE = ErrCode.EADDRINUSE
ERR_EADDRNOTAVAIL = ErrCode.EADDRNOTAVAIL
ERR_EADV = ErrCode.EADV
ERR_EAFNOSUPPORT = ErrCode.EAFNOSUPPORT
ERR_EAGAIN = ErrCode.EAGAIN
ERR_EALREADY = ErrCode.EALREADY
ERR_EAUTH = ErrCode.EAUTH
ERR_EBACKGROUND = ErrCode.EBACKGROUND
ERR_EBADE = ErrCode.EBADE
ERR_EBADF = ErrCode.EBADF
ERR_EBADFD = ErrCode.EBADFD
ERR_EBADMSG = ErrCode.EBADMSG
ERR_EBADR = ErrCode.EBADR
ERR_EBADRPC = ErrCode.EBADRPC
ERR_EBADRQC = ErrCode.EBADRQC
ERR_EBADSLT = ErrCode.EBADSLT
ERR_EBFONT = ErrCode.EBFONT
ERR_EBUSY = ErrCode.EBUSY
ERR_ECANCELED = ErrCode.ECANCELED
ERR_ECHILD = ErrCode.ECHILD
ERR_ECHRNG = ErrCode.ECHRNG
ERR_ECOMM = ErrCode.ECOMM
ERR_ECONNABORTED = ErrCode.ECONNABORTED
ERR_ECONNREFUSED = ErrCode.ECONNREFUSED
ERR_ECONNRESET = ErrCode.ECONNRESET
ERR_ED = ErrCode.ED
ERR_EDEADLK = ErrCode.EDEADLK
ERR_EDEADLOCK = ErrCode.EDEADLOCK
ERR_EDESTADDRREQ = ErrCode.EDESTADDRREQ
ERR_EDIED = ErrCode.EDIED
ERR_EDOM = ErrCode.EDOM
ERR_EDOTDOT = ErrCode.EDOTDOT
ERR_EDQUOT = ErrCode.EDQUOT
ERR_EEXIST = ErrCode.EEXIST
ERR_EFAULT = ErrCode.EFAULT
ERR_EFBIG = ErrCode.EFBIG
ERR_EFTYPE = ErrCode.EFTYPE
ERR_EGRATUITOUS = ErrCode.EGRATUITOUS
ERR_EGREGIOUS = ErrCode.EGREGIOUS
ERR_EHOSTDOWN = ErrCode.EHOSTDOWN
ERR_EHOSTUNREACH = ErrCode.EHOSTUNREACH
ERR_EIDRM = ErrCode.EIDRM
ERR_EIEIO = ErrCode.EIEIO
ERR_EILSEQ = ErrCode.EILSEQ
ERR_EINPROGRESS = ErrCode.EINPROGRESS
ERR_EINTR = ErrCode.EINTR
ERR_EINVAL = ErrCode.EINVAL
ERR_EIO = ErrCode.EIO
ERR_EISCONN = ErrCode.EISCONN
ERR_EISDIR = ErrCode.EISDIR
ERR_EISNAM = ErrCode.EISNAM
ERR_EL2HLT = ErrCode.EL2HLT
ERR_EL2NSYNC = ErrCode.EL2NSYNC
ERR_EL3HLT = ErrCode.EL3HLT
ERR_EL3RST = ErrCode.EL3RST
ERR_ELIBACC = ErrCode.ELIBACC
ERR_ELIBBAD = ErrCode.ELIBBAD
ERR_ELIBEXEC = ErrCode.ELIBEXEC
ERR_ELIBMAX = ErrCode.ELIBMAX
ERR_ELIBSCN = ErrCode.ELIBSCN
ERR_ELNRNG = ErrCode.ELNRNG
ERR_ELOOP = ErrCode.ELOOP
ERR_EMEDIUMTYPE = ErrCode.EMEDIUMTYPE
ERR_EMFILE = ErrCode.EMFILE
ERR_EMLINK = ErrCode.EMLINK
ERR_EMSGSIZE = ErrCode.EMSGSIZE
ERR_EMULTIHOP = ErrCode.EMULTIHOP
ERR_ENAMETOOLONG = ErrCode.ENAMETOOLONG
ERR_ENAVAIL = ErrCode.ENAVAIL
ERR_ENEEDAUTH = ErrCode.ENEEDAUTH
ERR_ENETDOWN = ErrCode.ENETDOWN
ERR_ENETRESET = ErrCode.ENETRESET
ERR_ENETUNREACH = ErrCode.ENETUNREACH
ERR_ENFILE = ErrCode.ENFILE
ERR_ENOANO = ErrCode.ENOANO
ERR_ENOBUFS = ErrCode.ENOBUFS
ERR_ENOCSI = ErrCode.ENOCSI
ERR_ENODATA = ErrCode.ENODATA
ERR_ENODEV = ErrCode.ENODEV
ERR_ENOENT = ErrCode.ENOENT
ERR_ENOEXEC = ErrCode.ENOEXEC
ERR_ENOLCK = ErrCode.ENOLCK
ERR_ENOLINK = ErrCode.ENOLINK
ERR_ENOMEDIUM = ErrCode.ENOMEDIUM
ERR_ENOMEM = ErrCode.ENOMEM
ERR_ENOMSG = ErrCode.ENOMSG
ERR_ENONET = ErrCode.ENONET
ERR_ENOPKG = ErrCode.ENOPKG
ERR_ENOPROTOOPT = ErrCode.ENOPROTOOPT
ERR_ENOSPC = ErrCode.ENOSPC
ERR_ENOSR = ErrCode.ENOSR
ERR_ENOSTR = ErrCode.ENOSTR
ERR_ENOSYS = ErrCode.ENOSYS
ERR_ENOTBLK = ErrCode.ENOTBLK
ERR_ENOTCONN = ErrCode.ENOTCONN
ERR_ENOTDIR = ErrCode.ENOTDIR
ERR_ENOTEMPTY = ErrCode.ENOTEMPTY
ERR_ENOTNAM = ErrCode.ENOTNAM
ERR_ENOTSOCK = ErrCode.ENOTSOCK
ERR_ENOTSUP = ErrCode.ENOTSUP
ERR_ENOTTY = ErrCode.ENOTTY
ERR_ENOTUNIQ = ErrCode.ENOTUNIQ
ERR_ENXIO = ErrCode.ENXIO
ERR_EOPNOTSUPP = ErrCode.EOPNOTSUPP
ERR_EOVERFLOW = ErrCode.EOVERFLOW
ERR_EPERM = ErrCode.EPERM
ERR_EPFNOSUPPORT = ErrCode.EPFNOSUPPORT
ERR_EPIPE = ErrCode.EPIPE
ERR_EPROCLIM = ErrCode.EPROCLIM
ERR_EPROCUNAVAIL = ErrCode.EPROCUNAVAIL
ERR_EPROGMISMATCH = ErrCode.EPROGMISMATCH
ERR_EPROGUNAVAIL = ErrCode.EPROGUNAVAIL
ERR_EPROTO = ErrCode.EPROTO
ERR_EPROTONOSUPPORT = ErrCode.EPROTONOSUPPORT
ERR_EPROTOTYPE = ErrCode.EPROTOTYPE
ERR_ERANGE = ErrCode.ERANGE
ERR_EREMCHG = ErrCode.EREMCHG
ERR_EREMOTE = ErrCode.EREMOTE
ERR_EREMOTEIO = ErrCode.EREMOTEIO
ERR_ERESTART = ErrCode.ERESTART
ERR_EROFS = ErrCode.EROFS
ERR_ERPCMISMATCH = ErrCode.ERPCMISMATCH
ERR_ESHUTDOWN = ErrCode.ESHUTDOWN
ERR_ESOCKTNOSUPPORT = ErrCode.ESOCKTNOSUPPORT
ERR_ESPIPE = ErrCode.ESPIPE
ERR_ESRCH = ErrCode.ESRCH
ERR_ESRMNT = ErrCode.ESRMNT
ERR_ESTALE = ErrCode.ESTALE
ERR_ESTRPIPE = ErrCode.ESTRPIPE
ERR_ETIME = ErrCode.ETIME
ERR_ETIMEDOUT = ErrCode.ETIMEDOUT
ERR_ETOOMANYREFS = ErrCode.ETOOMANYREFS
ERR_ETXTBSY = ErrCode.ETXTBSY
ERR_EUCLEAN = ErrCode.EUCLEAN
ERR_EUNATCH = ErrCode.EUNATCH
ERR_EUSERS = ErrCode.EUSERS
ERR_EWOULDBLOCK = ErrCode.EWOULDBLOCK
ERR_EXDEV = ErrCode.EXDEV
ERR_EXFULL = ErrCode.EXFULL
